#include "user.h"

/**
 * Global variables
 * float, int, and unsigned int, can be monitored and altered from imperix Cockpit
 */
float duty_cycle  = 0.3;

/**
 * Initialization routine executed only once, before the first call of the main interrupt
 * To be used to configure all needed peripherals and perform all needed initializations
 */
tUserSafe UserInit(void)
{
	/**
	 * Configuration of the main interrupt:
	 * - The frequency of CLOCK_0 is set to the desired switching frequency
	 * - The main interrupt routine is mapped on CLOCK_0, with a phase of 0
	 */
	Clock_SetFrequency(CLOCK_0, 50e3);
	ConfigureMainInterrupt(UserInterrupt, CLOCK_0, 0);

	/**
	 * Configuration of the SBO interface that will be used
	 * to send the duty-cycle value to the FPGA
	 * (uses two 16-bits registers to send a 32-bit float number)
	 */
	Sbo_ConfigureAsRealTime(0);
	Sbo_ConfigureAsRealTime(1);

	/**
	 * Configuration of the Sandbox PWM
	 * - channel 0 with complementary outputs
	 * - dead-time of 1us between the complementary outputs
	 */
	SbPwm_ConfigureOutputMode(PWM_CHANNEL_0, COMPLEMENTARY);
	SbPwm_ConfigureDeadTime(PWM_CHANNEL_0, DEADTIME);
	SbPwm_Activate(PWM_CHANNEL_0);

	return SAFE;
}


/**
 * Main interrupt routine
 */
tUserSafe UserInterrupt(void)
{
	uint16 sbo_0 = 0;
	uint16 sbo_1 = 0;

	/**
	 * Split duty-cycle value (32-bit float) into two 16-bit unsigned int
	 * As duty_cycle is a global variable, its value can be altered at runtime in Cockpit
	 */
	float2sbo(sbo_0, sbo_1, duty_cycle);

	/**
	 * Write both uint16 values into the SBO registers 0 and 1
	 * to send the duty-cycle value to the FPGA
	 */
	Sbo_Write(0, sbo_0);
	Sbo_Write(1, sbo_1);

	return SAFE;
}


/**
 * Routine executed when the core state goes into FAULT mode
 */
void UserError(tErrorSource source)
{

}

/**
 * Helper method that splits a float into two uint16
 * for passing float numbers through SBO registers
 */
void float2sbo(uint16 &sbo_0, uint16 &sbo_1, const float &float_in)
{
	uint16 temp[2];
	memcpy((void *)&temp[0], (void *)&float_in, (size_t)2 * sizeof(uint16));
	sbo_0 = temp[0];
	sbo_1 = temp[1];
}
